#!/usr/bin/python
# -*- coding: latin-1 -*-
# WAVELET Torrence and Combo translate from Matlab to Python
# author: Mabel Calim Costa
# INPE
# 23/01/2013

"Baseado : Torrence e Combo"

# data from http://paos.colorado.edu/research/wavelets/software.html

import numpy as np
import pylab
from pylab import *
import matplotlib.pyplot as plt
import os
import sys




def load_txt(archive,dt,date1):
	"""Archive .dat and .txt  without date 
           Call function:
			data,n,time = load_txt('sst_nino3.dat',0.25,1871)

  	"""
        pkgs_pth = os.path.join(sys.prefix, 'lib', 'python'+sys.version[:3],'site-packages','wavelet','lib','wavelet','data','txt',archive)
        data = np.loadtxt(pkgs_pth)
        n = len(data)
        time = np.arange(n)*dt + date1  
	return data,n,time
#data,n,time = load_txt('sst_nino3.dat',0.25,1871)
	
def normalize(data):
        """Call function:
			data_norm = normalize(data)
 	"""
	variance = np.var(data)
	data = (data - np.mean(data))/(np.sqrt(variance))
	return data, variance
#data_norm,variance = normalize(data)

def cwt (data, dt, variance, n, pad, dj,s0,j1,lag1,param,mother): 
	"""pad = 1         # pad the time series with zeroes (recommended)
	   dj = 0.25       # this will do 4 sub-octaves per octave
           s0 = 2*dt       # this says start at a scale of 6 months
           j1 = 7/dj       # this says do 7 powers-of-two with dj sub-octaves each
           lag1 = 0.72     # lag-1 autocorrelation for red noise background
           param = 6
           mother = 'Morlet'

        """
        
	# Wavelet transform
	import lib_wavelet
	ondaleta,wave,period,scale,coi = lib_wavelet.wavelet(data,dt,mother,param)#,pad,dj,s0,j1,mother,param)
	power = np.abs(wave*wave)

	# Significance levels: (variance=1 for the normalized SST)
	signif,fft_theor = lib_wavelet.wave_signif(1.0,dt,scale,0,lag1,0.95,-1,mother,param)
	ones = np.ones((len(signif),n))		# expand signif --> ones (J+1)x(n)
	sig95 = [s * ones[1] for s in signif] 	# vstack signif concatenate
	sig95 = power/sig95			# where ratio > 1, power is significant

	# Global wavelet spectrum & significance levels:
	global_ws = variance*(np.sum(power.conj().transpose(),axis=0)/n)
	dof  = [n-s for s in scale]
	"""CAUTION - DEFAULT VALUES """
	global_signif,fft_theor = lib_wavelet.wave_signif(variance,dt,scale,1,lag1,0.95,dof,mother,param)

	# Daughter wavelet
	joint_wavelet = np.concatenate((np.fft.ifft(ondaleta)[np.ceil(n/2.):],np.fft.ifft(ondaleta)[np.ceil(n/2.):][::-1]), axis =1) #joint Wavelet
	imag_wavelet = np.concatenate((np.fft.ifft(ondaleta).imag[np.ceil(n/2.):],np.fft.ifft(ondaleta).imag[np.ceil(n/2.):][::-1]), axis =1)
	nw = np.size(joint_wavelet)  # daughter's number of points 
	# admissibility condition
	mean_wavelet = mean(joint_wavelet.real)
	mean_wavelet = np.ones(nw)*mean_wavelet
        
        result = {'ondaleta':ondaleta,'wave':wave ,'period': period,'scale':scale ,'coi':coi,'power': power,'sig95':sig95, 'global_ws':global_ws,
			'global_signif':global_signif,'joint_wavelet':joint_wavelet,'imag_wavelet':imag_wavelet,'nw':nw ,'mean_wavelet':mean_wavelet ,}
	return result
 
#result = cwt(data_norm,0.25,variance,n,1,0.25,2*0.25,7/0.25,0.72,6,'Morlet')
# ---------------------------
#           Ploting 
# ---------------------------

def wavelet_plot(time,data,result):
        from numpy import log2
        import numpy as np

	fig = plt.figure(figsize=(15,10), dpi=100)
	ax = fig.add_subplot(411)
	"""Plot time series """
	subplot(3,1,1)
	plot(time,data)
	xlabel('Time (year)')
	ylabel('NINO3 SST (degC)')
	title('a) NINO3 Sea Surface Temperature (seasonal)')

	subplot(7,4,14)
	plot(range(-result['nw']/2,result['nw']/2),result['joint_wavelet'],'k')
	plot(range(-result['nw']/2,result['nw']/2),result['imag_wavelet'],'--k')
	plot(range(-result['nw']/2,result['nw']/2),result['mean_wavelet'],'g')
	xlim(-50,50)
	ylim(-0.2,0.2)
	text(20,0.10,'Morlet')
	title('$\psi$ (t/s)')

	subplot(7,4,15)
	plot(result['ondaleta'],'k') 
	title('$\psi^-$  ')

	""" Contour plot wavelet power spectrum """
	subplot(3,2,5)
	levels = [0.0625,0.125,0.25,0.5,1,2,4,8,16]
	contour(time, np.log2(result['period']),np.log2(result['power']),np.log2(levels))
	ax = gca()							# handle to the current axes
	ax.set_ylim(ax.get_ylim()[::-1])				# reverse plot along y axis
	yt =  range(int(np.log2(result['period'][0])),int(np.log2(result['period'][-1]))+1) # create the vector of periods
	Yticks = [int(math.pow(2,p)) for p in yt]			# make 2^periods
	yticks(yt, map(str,Yticks))
	xlim(time[0],time[-1])                  			# date on the x axis 
	# 95% significance contour, levels at -99 (fake) and 1 (95% signif)
	contour(time,np.log2(result['period']),result['sig95'], [-99,1],linewidths= 2)
	# cone-of-influence , anything "below"is dubious
	#coi = np.ma.masked_where(coi <= np.log2(power),coi)
	plot(time,np.log2(result['coi']),'k')
	xlabel('Time (year)')
	ylabel('Period (years)')
	title('b) NINO3 SST Wavelet Power Spectrum')


	""" Plot global wavelet spectrum"""
	subplot(3,2,6)
	plot(result['global_ws'], np.log2(result['period']))
	plot(result['global_signif'],np.log2(result['period']),'r--')
	ax = gca()                              # handle to the current axes
	ax.set_ylim(ax.get_ylim()[::-1])        # reverse plot along y axis
	yticks(yt, map(str,Yticks))
	#xlim(0,1.25*np.max(global_ws))
	xlabel('Power (deg C$^{2})$')
	title('c) Global Wavelet Spectrum')

	#plt.savefig('wavelet_nino.png')
	pylab.show()
        return

#wavelet_plot(time,data,result)



#!/usr/bin/python
# -*- coding: latin-1 -*-
# WAVELET LIBRARY - Based on Torrence and Combo (1998)
# author: Mabel Calim Costa
# GMAO - INPE
# 23/01/2013

import numpy as np
import pylab
from pylab import detrend_mean
import math

""" Translating mfiles of the Torrence and Combo to python functions
    1 - wavetest.m
    2 - wave_bases.m
    3 - wave_signif.m
    4 - chisquare_inv.m
    5 - chisquare_solve.m
"""
def nextpow2(i):
    n = 2
    while n < i: n = n * 2
    return n

def wave_bases(mother,k,scale,param):
        """Computes the wavelet function as a function of Fourier frequency
           used for the CWT in Fourier space (Torrence and Compo, 1998) 
           -- This def is called automatically by def wavelet --

           _____________________________________________________________________
           Inputs:
                  mother - a string equal to 'Morlet'
                  k      - a vectorm the Fourier frequecies 
                  scale  - a number, the wavelet scale
                  param  - the nondimensional parameter for the wavelet function

           Outputs:
                  daughter       - a vector, the wavelet function 
                  fourier_factor - the ratio os Fourier period to scale
                  coi            - a number, the cone-of-influence size at the scale
                  dofmin         - a number, degrees of freedom for each point in the wavelet power (Morlet = 2)
           
           Call function:
                  daughter,fourier_factor,coi,dofmin = wave_bases(mother,k,scale,param) 
           _____________________________________________________________________
        """
        n = len(k)							# length of Fourier frequencies (came from wavelet.py)
	"""CAUTION : default values""" 
        if (mother == 'Morlet'):					# choose the wavelet function, in this case Morlet
		param = 6 						# For Morlet this is k0 (wavenumber) default is 6
		k0 = param
        	expnt = -pow(scale*k-k0,2)/2*(k>0)			# table 1 Torrence and Compo (1998)
        	norm = math.sqrt(scale*k[1])*(pow(math.pi,-0.25))*math.sqrt(len(k))
        	daughter = []						# define daughter as a list
        	for ex in expnt:					# for each value scale (equal to next pow of 2)
            		daughter.append(norm*math.exp(ex))
   		k = np.array(k)						# turn k to array
        	daughter = np.array(daughter)				# transform in array
        	daughter = daughter*(k>0) 				# Heaviside step function
    		fourier_factor = (4*math.pi)/(k0+math.sqrt(2+k0*k0))    # scale --> Fourier 
    		coi = fourier_factor/math.sqrt(2)                       # cone-of- influence 
    		dofmin = 2					        # degrees of freedom	
        elif (mother == 'DOG'): 
        	param = 2   
        	m = param
        	expnt = -pow(scale*k,2)/2
                """CAUTION gamma(m+0.5) = 1.3293"""
         	norm = math.sqrt(scale*k[1]/1.3293*math.sqrt(n)) 
                daughter =[]
                for ex in expnt:
			daughter.append(-norm*pow(1j,m)*(pow(scale*k,m))*math.exp(ex))
                fourier_factor = 2*math.pi*math.sqrt(2/(2*m+1))
                coi = fourier_factor/math.sqrt(2)
                dofmin = 1 
    	return daughter,fourier_factor,coi,dofmin



def wavelet(Y,dt,mother,param):#,pad,dj,s0,J1,mother,param):
        """Computes the wavelet continuous transform of the vector Y, by definition:
        
           W(a,b) = sum(f(t)*psi[a,b](t) dt)               a dilate/contract 
           psi[a,b](t) = 1/sqrt(a) psi(t-b/a)              b displace 

           Only Morlet wavelet (k0=6) is used
           The wavelet basis is normalized to have total energy = 1 at all scales 
 
           _____________________________________________________________________
           Input:
           	Y - time series 
	   	dt - sampling rate
           	mother - the mother wavelet function
           	param - the mother wavelet parameter

           Output:
           	ondaleta - wavelet bases at scale 10 dt
           	wave - wavelet transform of Y
           	period - the vector of "Fourier"periods ( in time units) that correspond to the scales
           	scale - the vector of scale indices, given by S0*2(j*DJ), j =0 ...J1
           	coi - cone of influence   		  
      
           Call function:
           ondaleta, wave, period, scale, coi = wavelet(Y,dt,mother,param)  
           _____________________________________________________________________

        """ 
        # if(param == -1): param == -1
        # if ~exist(c,var) || isempty(c):  c = 10
        """CAUTION : default values"""  
	n1 = len(Y)							# time series length 
	s0 = 2*dt							# smallest scale of the wavelet
	dj = 0.25							# spacing between discrete scales
	J1= int(np.floor((np.log10(n1*dt/s0))/np.log10(2)/dj)) 		# J1+1 total os scales
	#mother = 'Morlet'
	# pad if necessary
	x = detrend_mean(Y)						#extract the mean of time series
	pad = 1
	if (pad ==1) :
    		base2 = nextpow2(n1)					#call det nextpow2
	n = base2			
        """CAUTION"""
        # construct wavenumber array used in transform
	# simetric eqn 5  
	k = np.arange(n/2)
	import math
	k_pos,k_neg=[],[]
	for i in range(0,n/2+1):
    		k_pos.append(i*((2*math.pi)/(n*dt)))			# frequencies as in eqn5
    		k_neg = k_pos[::-1]					# inversion vector
    		k_neg = [e * (-1) for e in k_neg]			# negative part 
    		k_neg = k_neg[1:-1]					# delete the first value of k_neg = last value of k_pos
	k = np.concatenate((k_pos,k_neg), axis =1)  			# vector of symmetric
	# compute fft of the padded time series
	f = np.fft.fft(x,n)
	scale=[]
	for i in range(J1+1):
    		scale.append(s0*pow(2,(i)*dj))
	period = scale
	wave = np.zeros((J1+1,n))					#define wavelet array
	wave = wave+1j * wave						# make it complex
	#loop through scales and compute transform
        for a1 in range(J1+1):
    		daughter,fourier_factor,coi,dofmin = wave_bases(mother,k,scale[a1],param)	#call wave_bases
    		#zf = np.zeros(len(f))
    		#zf[len(f)/2:len(f)*3/2] = daughter        		#wavelet in the middle of the zero vector
    		wave[a1,:] = np.fft.ifft(f*daughter)			# wavelet transform
                if a1==11 : ondaleta =daughter
	period = np.array(period)
        period = period[:]*fourier_factor
        #cone-of-influence
        mat = np.concatenate((range(1,n1/2),range(1,n1/2)[::-1]), axis=1)	# create mirrored array
        mat = np.insert(mat,0,0)						# insert zero at the begining of the array
        mat = np.append(mat,0)						# insert zero at the end of the array
        coi = [coi*dt*m for m in mat] 						# create coi matrix
        wave = wave[:,0:n1]
        return ondaleta, wave, period, scale, coi

def wave_signif(Y,dt,scale1,sigtest,lag1,sig1v1,dof,mother,param):
	"""CAUTION : default values"""
        import scipy
        from scipy import stats
	n1 = np.size(Y)
	J1 = len(scale1) -1
	s0 = np.min(scale1)
	dj = np.log10(scale1[1]/scale1[0])/np.log10(2)
	"""CAUTION"""
        if (n1 ==1): 
           variance = Y
        else: 
           variance =np.var(Y)
	"""CAUTION"""
	#sig1v1 = 0.95
        if (mother == 'Morlet'):
		#get the appropriate parameters [see table2]
		param = 6
		k0 = param
		fourier_factor = float(4*math.pi)/(k0 + np.sqrt(2+ k0*k0))
		empir = [2,-1,-1,-1]
		if(k0 == 6): empir[1:4] = [0.776,2.32,0.6]

        if(mother == 'DOG'):
		param = 2
	        fourier_factor = float(2*math.pi*np.sqrt(2/(2*m+1)))
		empir = [1,-1,-1,-1]
 		if(k0 == 2): empir[1:4] = [3.541, 1.43, 1.4]

	period = [e * fourier_factor for e in scale1]
	dofmin = empir[0]	# Degrees of  freedom with no smoothing
	Cdelta = empir[1]	# reconstruction factor
	gamma_fac = empir[2]	# time-decorrelation factor
	dj0 = empir[3]		# scale-decorrelation factor
	freq = [dt/p for p in period]
	fft_theor = [((1-lag1*lag1)/(1-2*lag1*np.cos(f*2*math.pi)+lag1*lag1)) for f in freq] 
	fft_theor = [variance * ft for ft in fft_theor]
	signif = fft_theor
	if(dof == -1): dof = dofmin
        """CAUTION"""
	if(sigtest == 0):  
		dof = dofmin
		chisquare = scipy.special.gammaincinv(dof/2.0,sig1v1)*2.0/dof
		signif = [ft*chisquare for ft in fft_theor]
        elif (sigtest == 1): 
             	"""CAUTION: if len(dof) ==1"""
             	dof = np.array(dof)
             	truncate = np.where(dof<1)
             	dof[truncate] = np.ones(np.size(truncate))
             	for i in range(len(scale1)):
                	 dof[i] = (dofmin*np.sqrt(1+pow((dof[i]*dt/gamma_fac/scale1[i]),2))) 
             	dof = np.array(dof) # has to be an array to use np.where
             	truncate = np.where (dof<dofmin)
             	dof[truncate] = [dofmin* n for n in np.ones(len(truncate))] #minimum DOF is dofmin
             	chisquare,signif =[],[]
             	for a1 in range(J1+1):
                	chisquare.append(scipy.special.gammaincinv(dof[a1]/2.0,sig1v1)*2.0/dof[a1])
               		signif.append(fft_theor[a1]*chisquare[a1])
             	"""CAUTION : missing elif(sigtest ==2)"""  
        return signif,fft_theor
 



# -*- coding: utf-8 -*-
"""
Created on Mon Jun 17 2013

@author: Mabel Calim Costa
"""

from __future__ import absolute_import

from .cwt.lib_wavelet import nextpow2
from .cwt.lib_wavelet import wave_bases
from .cwt.lib_wavelet import wavelet
from .cwt.lib_wavelet import wave_signif
from .cwt.wavetest    import load_txt
from .cwt.wavetest    import normalize
from .cwt.wavetest    import cwt
from .cwt.wavetest    import wavelet_plot

#from .cwt import wavetest

from .cwa.cross_wavelet import cross_wavelet





# Define the objects imported by imports of the form: from pyclimatetools import *
__all__ = ['nextpow2', 'wave_bases', 'wavelet', 'wave_signif','load_txt','normalize','cwt','wavelet_plot','cross_wavelet']

# Package version number.
__version__ = '0.0.1'



# -*- coding: utf-8 -*-
#!/usr/bin/python
# Cross Wavelet Analysis (CWA) based on Maraun and Kurths(2004).
# http://www.nonlin-processes-geophys.net/11/505/2004/npg-11-505-2004.pdf
# author: Mabel Calim Costa
# INPE
# 23/01/2013

"""
Created on Mon Jun 17 2013

@author: Mabel Calim Costa
"""



def cross_wavelet (wave1,wave2):
    """ Computes the cross wavelet analysis.
        A normalized time and scale resolved measure for the relationship 
        between two time series x1(ti) and x2(ti) is the wavelet coherency (WCO),
        which is defined as the amplitude of the WCS(wavelet cross spectrum) 
        normalized to the two single WPS(wavelet power spectrum) (Maraun and Kurths,2004).
         	WCOi(s) = |WCSi(s) |/ (WPSi1(s) WPSi2(s)) ˆ1/2 
        _____________________________________________________________________
        Inputs:  
		wave1 - wavelet transform of time series x1
                wave2 - wavelet transform of time series x2
        Outputs:
		cohere - wavelet coherency (WCO)
 	Call function:
		cohere = cross_wavelet(wave1,wave2)
    """ 
    cross_power = np.abs(wave1.real*wave2.imag)
    cohere = cross_power/np.sqrt(np.abs(wave1*wave1)* np.abs(wave2*wave2))
    return cohere




# -*- coding: utf-8 -*-
"""
Created on Mon Jun 17 14:03 2013

@author: Mabel Calim Costa
"""

import os
from setuptools import setup
#from distutils.core import setup

for line in open('lib/waipy/__init__.py').readlines():
    if line.startswith('__version__'):
        exec(line.strip())

setup(
    name = "waipy",
    description = ("Wavelet Analysis in Python"),
    version=__version__,
    author='Mabel Calim Costa',
    author_email='mabelcalim@gmail.com',
    url='https://wavelet-analysis.readthedocs.org/en/latest/index.html',
    long_description="""
This guide includes a Continuous Wavelet Transform (CWT), significance 
tests from based on Torrence and Compo (1998) and Cross Wavelet Analysis 
(CWA) based on Maraun and Kurths(2004).""",
    packages=['waipy', 'waipy.cwt', 'waipy.cwa' ],
    package_dir={'':'lib'}
)

